/****************************************************************************
 ** Deling Final Fantasy VIII Field Editor
 ** Copyright (C) 2009-2012 Arzel Jrme <myst6re@gmail.com>
 **
 ** This program is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****************************************************************************/
#include "JsmOpcode.h"

JsmOpcode::JsmOpcode() :
	op(0)
{

}

JsmOpcode::JsmOpcode(quint32 op) :
	op(op)
{
}

JsmOpcode::JsmOpcode(unsigned int key, int param)
{
	setKey(key);
	setParam(param);
}

unsigned int JsmOpcode::key() const
{
	if(hasParam()) 	return op >> 24;
	else			return op;
}

int JsmOpcode::param() const
{
	if(!(op & 0x00800000))		return op & 0x00FFFFFF;
	else						return op | 0xFF000000;
}

quint32 JsmOpcode::opcode() const
{
	return op;
}

void JsmOpcode::setOpcode(quint32 op)
{
	this->op = op;
}

void JsmOpcode::setKey(unsigned int key)
{
	if(hasParam()) 	op = (key << 24) | (param() & 0x00FFFFFF);
	else			op = key;
}

void JsmOpcode::setParam(int param)
{
	op = (key() << 24) | (param & 0x00FFFFFF);
}

bool JsmOpcode::hasParam() const
{
	return (op & 0xFF000000) != 0;
}

int JsmOpcode::popCount() const
{
	return -1;
}

int JsmOpcode::pushCount() const
{
	return -1;
}

QString JsmOpcode::name() const
{
	unsigned int k = key();
	return k<358 ? opcodes[k] : QString("UNKNOWN%1").arg(k-358+1);
}

QString JsmOpcode::paramStr() const
{
	return QString::number(param());
}

QString JsmOpcode::desc() const
{
	return QString();
}

JsmOpcodeCal::JsmOpcodeCal(quint32 op) : JsmOpcode(op) {}
JsmOpcodeCal::JsmOpcodeCal(unsigned int key, int param) : JsmOpcode(key, param) {}
bool JsmOpcodeCal::hasParam() const {	return true; }
int JsmOpcodeCal::popCount() const {	return 2; }
int JsmOpcodeCal::pushCount() const {	return 1; }
QString JsmOpcodeCal::paramStr() const {
	int p = param();
	return p<15 ? cal_table[p] : JsmOpcode::paramStr();
}

const char *JsmOpcodeCal::cal_table[15] = {
	"ADD",
	"SUB",
	"MUL",
	"MOD",
	"DIV",
	"MIN",
	"EQ",
	"GT",
	"GE",
	"LS",
	"LE",
	"NT",
	"AND",
	"OR",
	"EOR",
};

JsmOpcodePsh::JsmOpcodePsh(quint32 op) : JsmOpcode(op) {}
JsmOpcodePsh::JsmOpcodePsh(unsigned int key, int param) : JsmOpcode(key, param) {}
bool JsmOpcodePsh::hasParam() const {	return true; }
int JsmOpcodePsh::popCount() const {	return 0; }
int JsmOpcodePsh::pushCount() const {	return 1; }

JsmOpcodePop::JsmOpcodePop(quint32 op) : JsmOpcode(op) {}
JsmOpcodePop::JsmOpcodePop(unsigned int key, int param) : JsmOpcode(key, param) {}
bool JsmOpcodePop::hasParam() const {	return true; }
int JsmOpcodePop::popCount() const {	return 1; }
int JsmOpcodePop::pushCount() const {	return 0; }

/*
CAL: Un paramtre
JMP: Un paramtre (Sign ?)
JPF: Un paramtre (Sign ?)
LBL: Un paramtre
RET: Un paramtre
PSHN_L: Un paramtre (Sign ?)
PSHI_L: Un paramtre
POPI_L: Un paramtre
PSHM_B: Un paramtre
POPM_B: Un paramtre
PSHM_W: Un paramtre
POPM_W: Un paramtre
PSHM_L: Un paramtre
POPM_L: Un paramtre
PSHSM_B: Un paramtre
PSHSM_W: Un paramtre
PSHSM_L: Un paramtre
PSHAC: Un paramtre
REQ: Un paramtre
REQSW: Un paramtre
REQEW: Un paramtre
PREQ: Un paramtre
PREQSW: Un paramtre
PREQEW: Un paramtre
UNUSE: Un paramtre
HALT: Un paramtre
SET: Un paramtre
SET3: Un paramtre
IDLOCK: Un paramtre
IDUNLOCK: Un paramtre
EFFECTPLAY2: Un paramtre
FOOTSTEP: Un paramtre
JUMP: Un paramtre
JUMP3: Un paramtre
LADDERUP: Un paramtre
LADDERDOWN: Un paramtre
LADDERUP2: Un paramtre
LADDERDOWN2: Un paramtre
MAPJUMP: Un paramtre
MAPJUMP3: Un paramtre
SETMODEL: Un paramtre
BASEANIME: Un paramtre
ANIME: Un paramtre
ANIMEKEEP: Un paramtre
CANIME: Un paramtre
CANIMEKEEP: Un paramtre
RANIME: Un paramtre
RANIMEKEEP: Un paramtre
RCANIME: Un paramtre
RCANIMEKEEP: Un paramtre
RANIMELOOP: Un paramtre
RCANIMELOOP: Un paramtre
LADDERANIME: Un paramtre
DISCJUMP: Un paramtre
SETLINE: Aucun paramtre
LINEON: Aucun paramtre
LINEOFF: Aucun paramtre
WAIT: Aucun paramtre
MSPEED: Aucun paramtre
MOVE: Aucun paramtre
MOVEA: Aucun paramtre
PMOVEA: Aucun paramtre
CMOVE: Aucun paramtre
FMOVE: Aucun paramtre
PJUMPA: Aucun paramtre
ANIMESYNC: Aucun paramtre
ANIMESTOP: Aucun paramtre
MES: Aucun paramtre
MESSYNC: Aucun paramtre
MESVAR: Aucun paramtre
ASK: Aucun paramtre
WINSIZE: Aucun paramtre
WINCLOSE: Aucun paramtre
UCON: Aucun paramtre
UCOFF: Aucun paramtre
MOVIE: Aucun paramtre
MOVIESYNC: Aucun paramtre
SETPC: Aucun paramtre
DIR: Aucun paramtre
DIRP: Aucun paramtre
DIRA: Aucun paramtre
PDIRA: Aucun paramtre
SPUREADY: Aucun paramtre
TALKON: Aucun paramtre
TALKOFF: Aucun paramtre
PUSHON: Aucun paramtre
PUSHOFF: Aucun paramtre
ISTOUCH: Aucun paramtre
MAPJUMPO: Aucun paramtre
MAPJUMPON: Aucun paramtre
MAPJUMPOFF: Aucun paramtre
SETMESSPEED: Aucun paramtre
SHOW: Aucun paramtre
HIDE: Aucun paramtre
TALKRADIUS: Aucun paramtre
PUSHRADIUS: Aucun paramtre
AMESW: Aucun paramtre
AMES: Aucun paramtre
GETINFO: Aucun paramtre
THROUGHON: Aucun paramtre
THROUGHOFF: Aucun paramtre
BATTLE: Aucun paramtre
BATTLERESULT: Aucun paramtre
BATTLEON: Aucun paramtre
BATTLEOFF: Aucun paramtre
KEYSCAN: Aucun paramtre
KEYON: Aucun paramtre
AASK: Aucun paramtre
PGETINFO: Aucun paramtre
DSCROLL: Aucun paramtre
LSCROLL: Aucun paramtre
CSCROLL: Aucun paramtre
DSCROLLA: Aucun paramtre
LSCROLLA: Aucun paramtre
CSCROLLA: Aucun paramtre
SCROLLSYNC: Aucun paramtre
RMOVE: Aucun paramtre
RMOVEA: Aucun paramtre
RPMOVEA: Aucun paramtre
RCMOVE: Aucun paramtre
RFMOVE: Aucun paramtre
MOVESYNC: Aucun paramtre
CLEAR: Aucun paramtre
DSCROLLP: Aucun paramtre
LSCROLLP: Aucun paramtre
CSCROLLP: Aucun paramtre
LTURNR: Aucun paramtre
LTURNL: Aucun paramtre
CTURNR: Aucun paramtre
CTURNL: Aucun paramtre
ADDPARTY: Aucun paramtre
SUBPARTY: Aucun paramtre
CHANGEPARTY: Aucun paramtre
REFRESHPARTY: Aucun paramtre
SETPARTY: Aucun paramtre
ISPARTY: Aucun paramtre
ADDMEMBER: Aucun paramtre
SUBMEMBER: Aucun paramtre
LTURN: Aucun paramtre
CTURN: Aucun paramtre
PLTURN: Aucun paramtre
PCTURN: Aucun paramtre
JOIN: Aucun paramtre
BGANIME: Aucun paramtre
RBGANIME: Aucun paramtre
RBGANIMELOOP: Aucun paramtre
BGANIMESYNC: Aucun paramtre
BGDRAW: Aucun paramtre
BGOFF: Aucun paramtre
BGANIMESPEED: Aucun paramtre
SETTIMER: Aucun paramtre
DISPTIMER: Aucun paramtre
SETGETA: Aucun paramtre
SETROOTTRANS: Aucun paramtre
SETVIBRATE: Aucun paramtre
MOVIEREADY: Aucun paramtre
GETTIMER: Aucun paramtre
FADEIN: Aucun paramtre
FADEOUT: Aucun paramtre
FADESYNC: Aucun paramtre
SHAKE: Aucun paramtre
SHAKEOFF: Aucun paramtre
FADEBLACK: Aucun paramtre
FOLLOWOFF: Aucun paramtre
FOLLOWON: Aucun paramtre
GAMEOVER: Aucun paramtre
SHADELEVEL: Aucun paramtre
SHADEFORM: Aucun paramtre
FMOVEA: Aucun paramtre
FMOVEP: Aucun paramtre
SHADESET: Aucun paramtre
MUSICCHANGE: Aucun paramtre
MUSICLOAD: Aucun paramtre
FADENONE: Aucun paramtre
POLYCOLOR: Aucun paramtre
POLYCOLORALL: Aucun paramtre
KILLTIMER: Aucun paramtre
CROSSMUSIC: Aucun paramtre
DUALMUSIC: Aucun paramtre
EFFECTPLAY: Aucun paramtre
EFFECTLOAD: Aucun paramtre
LOADSYNC: Aucun paramtre
MUSICSTOP: Aucun paramtre
MUSICVOL: Aucun paramtre
MUSICVOLTRANS: Aucun paramtre
MUSICVOLFADE: Aucun paramtre
ALLSEVOL: Aucun paramtre
ALLSEVOLTRANS: Aucun paramtre
ALLSEPOSTRANS: Aucun paramtre
SEVOL: Aucun paramtre
SEVOLTRANS: Aucun paramtre
SEPOS: Aucun paramtre
SEPOSTRANS: Aucun paramtre
SETBATTLEMUSIC: Aucun paramtre
BATTLEMODE: Aucun paramtre
SESTOP: Aucun paramtre
INITSOUND: Aucun paramtre
BGSHADE: Aucun paramtre
BGSHADESTOP: Aucun paramtre
RBGSHADELOOP: Aucun paramtre
DSCROLL2: Aucun paramtre
LSCROLL2: Aucun paramtre
CSCROLL2: Aucun paramtre
DSCROLLA2: Aucun paramtre
CSCROLLA2: Aucun paramtre
SCROLLSYNC2: Aucun paramtre
SCROLLMODE2: Aucun paramtre
MENUENABLE: Aucun paramtre
MENUDISABLE: Aucun paramtre
FOOTSTEPON: Aucun paramtre
FOOTSTEPOFF: Aucun paramtre
FOOTSTEPOFFALL: Aucun paramtre
FOOTSTEPCUT: Aucun paramtre
PREMAPJUMP: Aucun paramtre
USE: Aucun paramtre
SPLIT: Aucun paramtre
ANIMESPEED: Aucun paramtre
RND: Aucun paramtre
DCOLADD: Aucun paramtre
DCOLSUB: Aucun paramtre
TCOLADD: Aucun paramtre
TCOLSUB: Aucun paramtre
FCOLADD: Aucun paramtre
FCOLSUB: Aucun paramtre
COLSYNC: Aucun paramtre
DOFFSET: Aucun paramtre
LOFFSETS: Aucun paramtre
COFFSETS: Aucun paramtre
LOFFSET: Aucun paramtre
COFFSET: Aucun paramtre
OFFSETSYNC: Aucun paramtre
RUNENABLE: Aucun paramtre
RUNDISABLE: Aucun paramtre
MAPFADEOFF: Aucun paramtre
MAPFADEON: Aucun paramtre
INITTRACE: Aucun paramtre
SETDRESS: Aucun paramtre
FACEDIR: Aucun paramtre
FACEDIRA: Aucun paramtre
FACEDIRP: Aucun paramtre
FACEDIRLIMIT: Aucun paramtre
FACEDIROFF: Aucun paramtre
SARALYOFF: Aucun paramtre
SARALYON: Aucun paramtre
SARALYDISPOFF: Aucun paramtre
SARALYDISPON: Aucun paramtre
MESMODE: Aucun paramtre
FACEDIRINIT: Aucun paramtre
FACEDIRI: Aucun paramtre
JUNCTION: Aucun paramtre
SETCAMERA: Aucun paramtre
BATTLECUT: Aucun paramtre
FOOTSTEPCOPY: Aucun paramtre
WORLDMAPJUMP: Aucun paramtre
RFACEDIR: Aucun paramtre
RFACEDIRA: Aucun paramtre
RFACEDIRP: Aucun paramtre
RFACEDIROFF: Aucun paramtre
FACEDIRSYNC: Aucun paramtre
COPYINFO: Aucun paramtre
RAMESW: Aucun paramtre
BGSHADEOFF: Aucun paramtre
AXIS: Aucun paramtre
MENUNORMAL: Aucun paramtre
MENUPHS: Aucun paramtre
BGCLEAR: Aucun paramtre
GETPARTY: Aucun paramtre
MENUSHOP: Aucun paramtre
DISC: Aucun paramtre
LSCROLL3: Aucun paramtre
CSCROLL3: Aucun paramtre
MACCEL: Aucun paramtre
MLIMIT: Aucun paramtre
ADDITEM: Aucun paramtre
SETWITCH: Aucun paramtre
SETODIN: Aucun paramtre
RESETGF: Aucun paramtre
MENUNAME: Aucun paramtre
REST: Aucun paramtre
MOVECANCEL: Aucun paramtre
ACTORMODE: Aucun paramtre
MENUSAVE: Aucun paramtre
SAVEENABLE: Aucun paramtre
PHSENABLE: Aucun paramtre
HOLD: Aucun paramtre
MOVIECUT: Aucun paramtre
SETPLACE: Aucun paramtre
SETDCAMERA: Aucun paramtre
CHOICEMUSIC: Aucun paramtre
GETCARD: Aucun paramtre
DRAWPOINT: Aucun paramtre
PHSPOWER: Aucun paramtre
KEY: Aucun paramtre
CARDGAME: Aucun paramtre
SETBAR: Aucun paramtre
DISPBAR: Aucun paramtre
KILLBAR: Aucun paramtre
SCROLLRATIO2: Aucun paramtre
WHOAMI: Aucun paramtre
MUSICSTATUS: Aucun paramtre
MUSICREPLAY: Aucun paramtre
DOORLINEOFF: Aucun paramtre
DOORLINEON: Aucun paramtre
MUSICSKIP: Aucun paramtre
DYING: Aucun paramtre
SETHP: Aucun paramtre
MOVEFLUSH: Aucun paramtre
MUSICVOLSYNC: Aucun paramtre
PUSHANIME: Aucun paramtre
POPANIME: Aucun paramtre
KEYSCAN2: Aucun paramtre
PARTICLEON: Aucun paramtre
PARTICLEOFF: Aucun paramtre
KEYSIGHNCHANGE: Aucun paramtre
ADDGIL: Aucun paramtre
ADDPASTGIL: Aucun paramtre
ADDSEEDLEVEL: Aucun paramtre
PARTICLESET: Aucun paramtre
SETDRAWPOINT: Aucun paramtre
MENUTIPS: Aucun paramtre
LASTIN: Aucun paramtre
LASTOUT: Aucun paramtre
SEALEDOFF: Aucun paramtre
MENUTUTO: Aucun paramtre
CLOSEEYES: Aucun paramtre
SETCARD: Aucun paramtre
HOWMANYCARD: Aucun paramtre
WHERECARD: Aucun paramtre
ADDMAGIC: Aucun paramtre
SWAP: Aucun paramtre
SPUSYNC: Aucun paramtre
BROKEN: Aucun paramtre
Unknown1: Aucun paramtre
Unknown2: Aucun paramtre
Unknown3: Aucun paramtre
Unknown4: Aucun paramtre
Unknown5: Aucun paramtre
Unknown6: Aucun paramtre
Unknown7: Aucun paramtre
Unknown8: Aucun paramtre
Unknown9: Aucun paramtre
Unknown10: Aucun paramtre
Unknown11: Aucun paramtre
Unknown12: Aucun paramtre
Unknown13: Aucun paramtre
Unknown14: Aucun paramtre
Unknown15: Aucun paramtre
Unknown16: Aucun paramtre
Unknown17: Aucun paramtre
Unknown18: Aucun paramtre
*/

const char *JsmOpcode::opcodes[376] = {
	"NOP",
	"CAL",
	"JMP",
	"JPF",
	"GJMP",
	"LBL",
	"RET",
	"PSHN_L",
	"PSHI_L",
	"POPI_L",
	"PSHM_B",
	"POPM_B",
	"PSHM_W",
	"POPM_W",
	"PSHM_L",
	"POPM_L",
	"PSHSM_B",
	"PSHSM_W",
	"PSHSM_L",
	"PSHAC",
	"REQ",
	"REQSW",
	"REQEW",
	"PREQ",
	"PREQSW",
	"PREQEW",
	"UNUSE",
	"DEBUG",
	"HALT",
	"SET",
	"SET3",
	"IDLOCK",
	"IDUNLOCK",
	"EFFECTPLAY2",
	"FOOTSTEP",
	"JUMP",
	"JUMP3",
	"LADDERUP",
	"LADDERDOWN",
	"LADDERUP2",
	"LADDERDOWN2",
	"MAPJUMP",
	"MAPJUMP3",
	"SETMODEL",
	"BASEANIME",
	"ANIME",
	"ANIMEKEEP",
	"CANIME",
	"CANIMEKEEP",
	"RANIME",
	"RANIMEKEEP",
	"RCANIME",
	"RCANIMEKEEP",
	"RANIMELOOP",
	"RCANIMELOOP",
	"LADDERANIME",
	"DISCJUMP",
	"SETLINE",
	"LINEON",
	"LINEOFF",
	"WAIT",
	"MSPEED",
	"MOVE",
	"MOVEA",
	"PMOVEA",
	"CMOVE",
	"FMOVE",
	"PJUMPA",
	"ANIMESYNC",
	"ANIMESTOP",
	"MESW",
	"MES",
	"MESSYNC",
	"MESVAR",
	"ASK",
	"WINSIZE",
	"WINCLOSE",
	"UCON",
	"UCOFF",
	"MOVIE",
	"MOVIESYNC",
	"SETPC",
	"DIR",
	"DIRP",
	"DIRA",
	"PDIRA",
	"SPUREADY",
	"TALKON",
	"TALKOFF",
	"PUSHON",
	"PUSHOFF",
	"ISTOUCH",
	"MAPJUMPO",
	"MAPJUMPON",
	"MAPJUMPOFF",
	"SETMESSPEED",
	"SHOW",
	"HIDE",
	"TALKRADIUS",
	"PUSHRADIUS",
	"AMESW",
	"AMES",
	"GETINFO",
	"THROUGHON",
	"THROUGHOFF",
	"BATTLE",
	"BATTLERESULT",
	"BATTLEON",
	"BATTLEOFF",
	"KEYSCAN",
	"KEYON",
	"AASK",
	"PGETINFO",
	"DSCROLL",
	"LSCROLL",
	"CSCROLL",
	"DSCROLLA",
	"LSCROLLA",
	"CSCROLLA",
	"SCROLLSYNC",
	"RMOVE",
	"RMOVEA",
	"RPMOVEA",
	"RCMOVE",
	"RFMOVE",
	"MOVESYNC",
	"CLEAR",
	"DSCROLLP",
	"LSCROLLP",
	"CSCROLLP",
	"LTURNR",
	"LTURNL",
	"CTURNR",
	"CTURNL",
	"ADDPARTY",
	"SUBPARTY",
	"CHANGEPARTY",
	"REFRESHPARTY",
	"SETPARTY",
	"ISPARTY",
	"ADDMEMBER",
	"SUBMEMBER",
	"ISMEMBER",
	"LTURN",
	"CTURN",
	"PLTURN",
	"PCTURN",
	"JOIN",
	"MESFORCUS",
	"BGANIME",
	"RBGANIME",
	"RBGANIMELOOP",
	"BGANIMESYNC",
	"BGDRAW",
	"BGOFF",
	"BGANIMESPEED",
	"SETTIMER",
	"DISPTIMER",
	"SHADETIMER",
	"SETGETA",
	"SETROOTTRANS",
	"SETVIBRATE",
	"STOPVIBRATE",
	"MOVIEREADY",
	"GETTIMER",
	"FADEIN",
	"FADEOUT",
	"FADESYNC",
	"SHAKE",
	"SHAKEOFF",
	"FADEBLACK",
	"FOLLOWOFF",
	"FOLLOWON",
	"GAMEOVER",
	"ENDING",
	"SHADELEVEL",
	"SHADEFORM",
	"FMOVEA",
	"FMOVEP",
	"SHADESET",
	"MUSICCHANGE",
	"MUSICLOAD",
	"FADENONE",
	"POLYCOLOR",
	"POLYCOLORALL",
	"KILLTIMER",
	"CROSSMUSIC",
	"DUALMUSIC",
	"EFFECTPLAY",
	"EFFECTLOAD",
	"LOADSYNC",
	"MUSICSTOP",
	"MUSICVOL",
	"MUSICVOLTRANS",
	"MUSICVOLFADE",
	"ALLSEVOL",
	"ALLSEVOLTRANS",
	"ALLSEPOS",
	"ALLSEPOSTRANS",
	"SEVOL",
	"SEVOLTRANS",
	"SEPOS",
	"SEPOSTRANS",
	"SETBATTLEMUSIC",
	"BATTLEMODE",
	"SESTOP",
	"BGANIMEFLAG",
	"INITSOUND",
	"BGSHADE",
	"BGSHADESTOP",
	"RBGSHADELOOP",
	"DSCROLL2",
	"LSCROLL2",
	"CSCROLL2",
	"DSCROLLA2",
	"LSCROLLA2",
	"CSCROLLA2",
	"DSCROLLP2",
	"LSCROLLP2",
	"CSCROLLP2",
	"SCROLLSYNC2",
	"SCROLLMODE2",
	"MENUENABLE",
	"MENUDISABLE",
	"FOOTSTEPON",
	"FOOTSTEPOFF",
	"FOOTSTEPOFFALL",
	"FOOTSTEPCUT",
	"PREMAPJUMP",
	"USE",
	"SPLIT",
	"ANIMESPEED",
	"RND",
	"DCOLADD",
	"DCOLSUB",
	"TCOLADD",
	"TCOLSUB",
	"FCOLADD",
	"FCOLSUB",
	"COLSYNC",
	"DOFFSET",
	"LOFFSETS",
	"COFFSETS",
	"LOFFSET",
	"COFFSET",
	"OFFSETSYNC",
	"RUNENABLE",
	"RUNDISABLE",
	"MAPFADEOFF",
	"MAPFADEON",
	"INITTRACE",
	"SETDRESS",
	"GETDRESS",
	"FACEDIR",
	"FACEDIRA",
	"FACEDIRP",
	"FACEDIRLIMIT",
	"FACEDIROFF",
	"SARALYOFF",
	"SARALYON",
	"SARALYDISPOFF",
	"SARALYDISPON",
	"MESMODE",
	"FACEDIRINIT",
	"FACEDIRI",
	"JUNCTION",
	"SETCAMERA",
	"BATTLECUT",
	"FOOTSTEPCOPY",
	"WORLDMAPJUMP",
	"RFACEDIRI",
	"RFACEDIR",
	"RFACEDIRA",
	"RFACEDIRP",
	"RFACEDIROFF",
	"FACEDIRSYNC",
	"COPYINFO",
	"PCOPYINFO",
	"RAMESW",
	"BGSHADEOFF",
	"AXIS",
	"AXISSYNC",
	"MENUNORMAL",
	"MENUPHS",
	"BGCLEAR",
	"GETPARTY",
	"MENUSHOP",
	"DISC",
	"DSCROLL3",
	"LSCROLL3",
	"CSCROLL3",
	"MACCEL",
	"MLIMIT",
	"ADDITEM",
	"SETWITCH",
	"SETODIN",
	"RESETGF",
	"MENUNAME",
	"REST",
	"MOVECANCEL",
	"PMOVECANCEL",
	"ACTORMODE",
	"MENUSAVE",
	"SAVEENABLE",
	"PHSENABLE",
	"HOLD",
	"MOVIECUT",
	"SETPLACE",
	"SETDCAMERA",
	"CHOICEMUSIC",
	"GETCARD",
	"DRAWPOINT",
	"PHSPOWER",
	"KEY",
	"CARDGAME",
	"SETBAR",
	"DISPBAR",
	"KILLBAR",
	"SCROLLRATIO2",
	"WHOAMI",
	"MUSICSTATUS",
	"MUSICREPLAY",
	"DOORLINEOFF",
	"DOORLINEON",
	"MUSICSKIP",
	"DYING",
	"SETHP",
	"GETHP",
	"MOVEFLUSH",
	"MUSICVOLSYNC",
	"PUSHANIME",
	"POPANIME",
	"KEYSCAN2",
	"KEYON2",
	"PARTICLEON",
	"PARTICLEOFF",
	"KEYSIGHNCHANGE",
	"ADDGIL",
	"ADDPASTGIL",
	"ADDSEEDLEVEL",
	"PARTICLESET",
	"SETDRAWPOINT",
	"MENUTIPS",
	"LASTIN",
	"LASTOUT",
	"SEALEDOFF",
	"MENUTUTO",
	"OPENEYES",
	"CLOSEEYES",
	"BLINKEYES",
	"SETCARD",
	"HOWMANYCARD",
	"WHERECARD",
	"ADDMAGIC",
	"SWAP",
	"SETPARTY2",
	"SPUSYNC",
	"BROKEN",
	"UNKNOWN1",
	"UNKNOWN2",
	"UNKNOWN3",
	"UNKNOWN4",
	"UNKNOWN5",
	"UNKNOWN6",
	"UNKNOWN7",
	"UNKNOWN8",
	"UNKNOWN9",
	"UNKNOWN10",
	"UNKNOWN11",
	"UNKNOWN12",
	"UNKNOWN13",
	"UNKNOWN14",
	"UNKNOWN15",
	"UNKNOWN16",
	"UNKNOWN17",
	"UNKNOWN18"
};
